/*
 * Copyright (c) 2008, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef RVIZ_INTERACTION_TOOL_H
#define RVIZ_INTERACTION_TOOL_H

#include <stdint.h>

#include <ros/subscriber.h>

#include <rviz/interactive_object.h>

#include "move_tool.h"

namespace rviz
{
class BoolProperty;

class InteractionTool : public Tool
{
  Q_OBJECT
public:
  InteractionTool();
  ~InteractionTool() override;

  void onInitialize() override;

  void activate() override;
  void deactivate() override;

  int processMouseEvent(ViewportMouseEvent& event) override;
  int processKeyEvent(QKeyEvent* event, RenderPanel* panel) override;

public Q_SLOTS:

  void hideInactivePropertyChanged(){};

protected:
  /** @brief Check if the mouse has moved from one object to another,
   * and update focused_object_ if so. */
  void updateFocus(const ViewportMouseEvent& event);

  /** @brief The object (control) which currently has the mouse focus. */
  InteractiveObjectWPtr focused_object_;

  uint64_t last_selection_frame_count_;

  MoveTool move_tool_;

  BoolProperty* hide_inactive_property_;
};

} // namespace rviz

#endif
